﻿using CompactGuitar.SongDatabase;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CompactGuitar
{
    public partial class SongDbForm : Form
    {
        SongDbManager sdm;
        public SongDbForm()
        {
            InitializeComponent();        
        }

        private async void SongDbForm_Load(object sender, EventArgs e)
        {
            sdm = new SongDbManager();
            DataPanel.BringToFront();
            DisplayAllSongs(await sdm.GetAllSongs());           
        }

        private void DisplayAllSongs(IEnumerable<Song> songs)
        {
            foreach (Song s in songs)
            {
                var songTable = CreateSongTable(s.Name, s.Author);
                DataPanel.Controls.Add(songTable);
                songTable.BringToFront();
            }
            //panel1.Controls.SetChildIndex(headTable, SongCount);
        }

        private void DisplayOneSong(Song song)
        {
            var songTable = CreateSongTable(song.Name, song.Author);
            DataPanel.Controls.Add(songTable);
            songTable.BringToFront();
        }

        private TableLayoutPanel CreateSongTable(string name, string author)
        {
            var table = new TableLayoutPanel { Dock = DockStyle.Top, Size = new Size(800, 44), RowCount = 1, ColumnCount = 1 };
            var text1 = new Label { Dock = DockStyle.Fill, TextAlign = ContentAlignment.MiddleCenter, BorderStyle = BorderStyle.FixedSingle, Cursor = Cursors.Hand };
            text1.Font = new Font("Microsoft Sans Serif", 14F, FontStyle.Bold);
            text1.Text = name;
            text1.Click += new EventHandler(InspectSong);
            table.Controls.Add(text1);
            return table;
        }

        private async void inspectBackButton_Click(object sender, EventArgs e)
        {
            AddSongPanel.BringToFront();
        }

        private void ImportTextToBox(TextBox box)
        {
            openFileDialog1.Multiselect = false;
            openFileDialog1.Filter = "TXT files|*.txt";
            DialogResult dr = openFileDialog1.ShowDialog();
            if (dr == DialogResult.OK)
            {
                var path = openFileDialog1.FileName;
                box.Text = File.ReadAllText(path);
            }
        }

        private void openFileButton1_Click(object sender, EventArgs e)
        {
            ImportTextToBox(chordsBox);
        }

        private void openFileButton2_Click(object sender, EventArgs e)
        {
            ImportTextToBox(lyricsBox);
        }

        private async void accept_Click(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(nameBox.Text))
            {
                warning.Text = "Name is required!";
                return;
            }
            var song = new Song { Name = nameBox.Text, Author = authorBox.Text, Chords = chordsBox.Text, Lyrics = lyricsBox.Text };
            await sdm.AddSong(song);
            DisplayOneSong(song);
            DataPanel.BringToFront();
            ClearAddPanel();
        }

        private void ClearAddPanel()
        {
            nameBox.Text = String.Empty;
            authorBox.Text = String.Empty;
            lyricsBox.Text = String.Empty;
            chordsBox.Text = String.Empty;
            warning.Text = String.Empty;
        }

        private void addSongBack_Click(object sender, EventArgs e)
        {
            ClearAddPanel();
            DataPanel.BringToFront();
        }

        private async void InspectSong(object sender, EventArgs e)
        {
            var label = (Label) sender;
            var songs = await sdm.GetAllSongs();
            var song = songs.Find(song => song.Name == label.Text);
            inspectName.Text = song.Name;
            inspectAuthor.Text = song.Author;
            inspectLyrics.Text = song.Lyrics;
            inspectChords.Text = song.Chords;
            InspectSongPanel.BringToFront();
        }

        private void button7_Click(object sender, EventArgs e)
        {
            DataPanel.BringToFront();
        }

        private async void deleteButton_Click(object sender, EventArgs e)
        {
            DecisionForm df = new DecisionForm("Are you sure you want to delete song from database?");
            df.ShowDialog();
            if (df.Exit == true)
            {
                foreach (Control control in DataPanel.Controls)
                {
                    var ekk = nameBox.Text;
                    if (control is TableLayoutPanel && control.Controls[0].Text == inspectName.Text)
                    {
                        DataPanel.Controls.Remove(control);
                        break;
                    }
                }           
                var songs = await sdm.GetAllSongs();
                var song = songs.Find(song => song.Name == inspectName.Text);
                await sdm.DeleteSong(song);
                DataPanel.BringToFront();
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private async void deleteDbButton_Click(object sender, EventArgs e)
        {
            DecisionForm df = new DecisionForm("Are you sure you want to delete this database?");
            df.ShowDialog();
            if (df.Exit == true)
            {
                await sdm.DeleteDatabase();
                Close();
            }
        }
    }
}
