﻿using System.Media;

namespace CompactGuitar
{
    partial class Form1
    {
        private string[][] Chords = new string[][] { 

            new string[] {
            "C Major", "C Minor", "D Major", "D Minor", 
            "E Major", "E Minor", "F Major", "F Minor", 
            "G Major", "G Minor", "A Major", "A Minor", 
            "B Major", "B Minor" },
            new string[] {
            "C# Major", "C# Minor", "D# Major", "D# Minor", 
            "F# Major", "F# Minor", "G# Major", "G# Minor", 
            "A# Major", "A# Minor"} };

        private int ChordsIndex;
        private string Folder;

        private GameLayout GL;

        private string RightChordName;

        private System.Windows.Forms.Timer GameTimer = new System.Windows.Forms.Timer();
        private float GameTimerCounter = 30.0f;

        private int MaxScore = 0;
        private int Score = 0;

        public void StartGame()
        {
            Random random = new Random();
            RightChordName = Chords[ChordsIndex][random.Next(0, Chords[ChordsIndex].Count() - 1)];
            var path = Directory.GetCurrentDirectory() + @"\..\..\..\Resources\" + Folder + @"\" + RightChordName;
            var rightOption = random.Next(0, 3);
            string name;
            var usedName = "";
            for (int i = 0; i < 3; i++)
            {
                if (i == rightOption)
                {
                    GL.Options[i].Text = RightChordName;
                }
                else
                {
                    do
                    {
                        name = Chords[ChordsIndex][random.Next(0, Chords[ChordsIndex].Count() - 1)];
                    } while (name == RightChordName || name == usedName);
                    GL.Options[i].Text = name;
                    usedName = name;
                }
            }
            if (GL.GameTable == ChordsGame)
                GuessChord.ImageLocation = path + ".png";
            else
                PlaySound(RightChordName, RightChordName[1]);

            GL.GameTable.BringToFront();
        }

        public async void ChooseOption(Button option)
        {
            if (option.Text != RightChordName)
            {
                PlaySound("errorSound", '.');
                option.BackColor = Color.Red;               
                SetBonuses(option, "-2 seconds");
                option.Enabled = false;
                TimerTickPlus(2.0f);
                return;
            }

            PlaySound(option.Text, option.Text[1]);
            option.BackColor = Color.Green;            
            Score++;
            GL.Score.Text = String.Format("Score: {0}", Score);
            SetBonuses(option, "+1 seconds");
            TimerTickPlus(-1.0f);
            ChangeTable(false);
            await Task.Delay(1000);
            ChangeTable(true);
            ResetColorsAndBonuses();
            StartGame();

        }

        private void PlaySound(string name, char folderChar = '.')
        {
            SoundPlayer chordSound;
            string folder;
            switch (folderChar)
            {
                case ' ':
                    folder = "ChordsSounds";
                    break;
                case '#':
                    folder = "ChordsSoundsSharp";
                    break;
                default:
                    folder = ".";
                    break;
            }
            chordSound = new SoundPlayer(Directory.GetCurrentDirectory() + @"\..\..\..\Resources\" + folder + @"\" + name + ".wav");
            chordSound.Play();
        }

        private void ChangeTable(bool status)
        {
            if (GL.GameTable == ChordsGame)
                ChordsGame.Enabled = status;
            else
                EarGame.Enabled = status;
        }

        private void ResetColorsAndBonuses()
        {
            for (int i = 0; i < 3; i++)
            {
                GL.Options[i].BackColor = SystemColors.Control;
                GL.Options[i].Enabled = true;
                GL.Bonuses[i].Text = String.Empty;
            }
        }

        private void ResetAll()
        {
            ChangeTable(true);
            ResetColorsAndBonuses();
            GameTimer.Tick -= new EventHandler(TimerTick);
            GameTimerCounter = 30.0f;
            Score = 0;
        }

        private void SetBonuses(Button option, string bonus)
        {
            if (option == GL.Options[0])
                GL.Bonuses[0].Text = bonus;
            else if (option == GL.Options[1])
                GL.Bonuses[1].Text = bonus + " seconds";
            else
                GL.Bonuses[2].Text = bonus;
        }

        private void InitializeGame(GameLayout gl, int chordsIndex, string folder)
        {
            GL = gl;
            ChordsIndex = chordsIndex;
            Folder = folder;
            GameTimer.Tick += new EventHandler(TimerTick);
            GameTimer.Interval = 100;
            GameTimer.Start();
            GL.Countdown.Text = String.Format("Time: {0:F1}", GameTimerCounter);
            GL.Score.Text = String.Format("Score: {0}", Score);
            GL.HighScore.Text = String.Format("High Score: {0}", MaxScore);
        }

        private void TimerTick(object sender, EventArgs e)
        {
            GameTimerCounter -= 0.1f;
            CheckTime();
        }

        private void TimerTickPlus(float bonus)
        {
            GameTimerCounter -= bonus;
            CheckTime();
        }

        private void CheckTime()
        {                  
            if (GameTimerCounter <= 0.00f)
            {
                PlaySound("ringing", '.');
                EndChordsGame();
                return;
            }             
            GL.Countdown.Text = String.Format("Time: {0:F1}", GameTimerCounter);
        }


        private void EndPrematurely()
        {
            GameTimer.Stop();
            DecisionForm end = new DecisionForm("Are you sure you want to quit?");
            end.ShowDialog();
            if (end.Exit)
            {
                ResetAll();
                BringTableToFront();
                return;
            }           
            GameTimer.Start();
        }

        private void BringTableToFront()
        {
            if (GL.GameTable == ChordsGame)
                ChordsMenu.BringToFront();
            else
                EarMenu.BringToFront();
        }

        private void EndChordsGame()
        {
            GameTimer.Stop();
            GL.Countdown.Text = String.Format("Time: 0,0");
            DecisionForm end;
            Task.Run(() => UpdateStats());
            if (Score > MaxScore)
            {
                SetMax();
                end = new DecisionForm(Score, true);
                MaxScore = Score;
            }
            else
            {
                end = new DecisionForm(Score, false);
            } 
            end.ShowDialog();
            ResetAll();
            if (end.Exit)
            {
                BringTableToFront();              
                return;
            }
            InitializeGame(GL, ChordsIndex, Folder);
            StartGame();
        }

        private void SetMax()
        {
            if (Folder == "Chords")
            {
                if (GL.GameTable == ChordsGame)
                    maxBasic.Text = Score.ToString();
                else
                    maxSharp.Text = Score.ToString();
            }
            else
            {
                if (GL.GameTable == ChordsGame)
                    maxBasic2.Text = Score.ToString();
                else
                    maxSharp2.Text = Score.ToString();
            }
        }

        private void UpdateStats()
        {
            List<string> lines;
            string t;
            if (GL.GameTable == ChordsGame)
                t = "Chord";                
            else
                t = "Ear";

            lines = File.ReadAllLines(Directory.GetCurrentDirectory() + @"\..\..\..\Resources\GameStats\" + t + "Stats.txt").ToList();

            lines = lines.Where(l => l.Substring(0, nickBox.Text.Length) != nickBox.Text).ToList();
            lines.Add(nickBox.Text + ": " + Score);
            lines = lines.OrderByDescending(l => Int32.Parse(l.Substring(l.IndexOf(l.First(c => char.IsWhiteSpace(c))) + 1))).ToList();
            File.WriteAllLines(Directory.GetCurrentDirectory() + @"\..\..\..\Resources\GameStats\" + t + "Stats.txt", lines.ToArray());
        }
    }
}
