﻿using NAudio.Wave;

namespace CompactGuitar.Tuner
{
    public partial class SoundDetectForm : Form
    {
        BufferedWaveProvider bufferedWaveProvider;
        Dictionary<string, float> noteBaseFreqs = new Dictionary<string, float>()        

            {
                { "E2", 82.41f },
                { "A2", 110.00f },
                { "D3", 146.83f },
                { "G3", 196.00f },
                { "B3", 246.94f },
                { "E4", 329.63f },

            };

        string CheckedString = "E2";
        bool Stop = false;
        
        public SoundDetectForm()
        {
            InitializeComponent();
        }

        private void SoundDetectForm_Load(object sender, EventArgs e)
        {
            InitializeDeviceBox();
        }

        private void SoundDetectForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            stopButton_Click(sender, e);
            Close();
        }


        public void InitializeDeviceBox()
        {
            int DCount = WaveInEvent.DeviceCount;

            for (int i = 0; i < DCount; i++)
            {
                deviceBox.Items.Add((i + ". " + WaveInEvent.GetCapabilities(i).ProductName));
            }
        }

        public async Task StartDetect()
        {
            WaveInEvent waveEvent = new WaveInEvent();
            waveEvent.DeviceNumber = deviceBox.SelectedIndex;
            waveEvent.WaveFormat = new WaveFormat(44100, 1);
            waveEvent.DataAvailable += WaveIn_DataAvailable;

            bufferedWaveProvider = new BufferedWaveProvider(waveEvent.WaveFormat);

            waveEvent.StartRecording();

            IWaveProvider stream = new Wave16ToFloatProvider(bufferedWaveProvider);
            SoundDetect sd = new SoundDetect(stream);

            byte[] buffer = new byte[8192];
            int bytes;

            await Task.Run(() =>
            {
                do
                {                   
                    bytes = stream.Read(buffer, 0, buffer.Length);

                    float freq = sd.GetFreq(buffer);

                    if (freq != 0)
                    {
                        freqLabel.Invoke((Action)delegate
                        {
                            freqLabel.Text = "Freq: " + freq;
                        });

                        float stringFreq;
                        noteBaseFreqs.TryGetValue(CheckedString, out stringFreq);

                        tuneLabel.Invoke((Action)delegate
                        {                           
                            if (stringFreq > freq)
                                tuneLabel.Text = "- " + (stringFreq - freq).ToString();
                            else
                                tuneLabel.Text = "+ " + (freq - stringFreq).ToString();
                            SetColorByDifference(Math.Abs(stringFreq - freq));
                        });
                        
                    }

                } while (bytes != 0 && !Stop);

                waveEvent.StopRecording();
                waveEvent.Dispose();
                Stop = false;
            });                     
        }

        private void SetColorByDifference(float diff)
        {
            if (diff < 1)
            {
                tuneLabel.ForeColor = Color.Green;
                checkPic.Visible = true;
            }              
            else if (diff < 3)
            {
                tuneLabel.ForeColor = Color.Orange;
                checkPic.Visible = false;
            }
            else
            {
                tuneLabel.ForeColor = Color.Red;
                checkPic.Visible = false;
            }                         
        }

        void WaveIn_DataAvailable(object sender, WaveInEventArgs e)
        {
            if (bufferedWaveProvider != null)
            {
                bufferedWaveProvider.AddSamples(e.Buffer, 0, e.BytesRecorded);
                bufferedWaveProvider.DiscardOnBufferOverflow = true;
            }
        }

        private async void detectButton_Click(object sender, EventArgs e)
        {
            if (deviceBox.SelectedIndex != -1)
            {
                detectButton.Enabled = false;
                msgLabel1.Text = "Play or sing a note!";
                await StartDetect();
            }
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            Stop = true;
            detectButton.Enabled = true;
            msgLabel1.Text = String.Empty;
            freqLabel.Text = String.Empty;
            tuneLabel.Text = String.Empty;
            checkPic.Visible = false;
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            CheckedString = radioButton1.Text.Substring(0, 2);
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            CheckedString = radioButton2.Text.Substring(0, 2); 
        }

        private void radioButton3_CheckedChanged(object sender, EventArgs e)
        {
            CheckedString = radioButton3.Text.Substring(0, 2);
        }

        private void radioButton4_CheckedChanged(object sender, EventArgs e)
        {
            CheckedString = radioButton4.Text.Substring(0, 2);
        }

        private void radioButton5_CheckedChanged(object sender, EventArgs e)
        {
            CheckedString = radioButton5.Text.Substring(0, 2);
        }

        private void radioButton6_CheckedChanged(object sender, EventArgs e)
        {
            CheckedString = radioButton6.Text.Substring(0, 2);
        }

        private void exitButton_Click(object sender, EventArgs e)
        {
            stopButton_Click(sender, e);
            this.Close();
        }

    }
}
